#!/usr/bin/env python3
# NOTE: part 2 of example caproto_to_ophyd.py
'''
Running this script assumes you have that IOC running initially.

This script will connect to that server through ophyd, initialize pydm, and
spawn an autogenerated typhon screen for the device.
'''
import sys

try:
    import pydm
    from typhon import DeviceDisplay
except ImportError as ex:
    print(f'pydm/typhon unavailable: {ex}')
    pydm = None

from caproto.ioc_examples.caproto_to_ophyd import GroupDevice


def run_typhon(prefix='integration:'):
    # See Steps 1...3 in caproto_to_ophyd.py
    dev = GroupDevice(prefix=prefix, name='dev')
    print(dev, dev.read_attrs)
    dev.wait_for_connection()
    get_value = dev.get()

    print('Current values are:')
    print(get_value)

    # Step 4 make a GUI in a line or two
    if pydm is not None:
        app = pydm.PyDMApplication()
        typhon_display = DeviceDisplay(dev)
        typhon_display.method_panel.add_method(dev.get_random.call)
        typhon_display.show()
        app.exec_()


if __name__ == '__main__':
    try:
        prefix = sys.argv[1]
    except IndexError:
        prefix = 'integration:'
    run_typhon(prefix=prefix)
